'use strict';

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.tasks = exports.task = exports.coffee2 = exports.babel = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _coffeescript = _interopRequireDefault(require("coffeescript"));
var _path2 = _interopRequireDefault(require("path"));
var _rollupPluginutils = require("rollup-pluginutils");
var _fs = _interopRequireDefault(require("fs"));
var _babylon = require("babylon");
var _traverse = _interopRequireDefault(require("@babel/traverse"));
var _generator = _interopRequireDefault(require("@babel/generator"));
var _types = require("@babel/types");
var _cfx = _interopRequireDefault(require("cfx.babel"));
var _del = _interopRequireDefault(require("del"));
var _gulpRename = _interopRequireDefault(require("gulp-rename"));
var _gulpBetterRollup = _interopRequireDefault(require("gulp-better-rollup"));
var _rollupPluginCleanup = _interopRequireDefault(require("rollup-plugin-cleanup"));
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { (0, _defineProperty2["default"])(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
function formatError(source, filename, err) {
  var carets, col, first_column, first_line, l, last_column, last_line, lines;
  var _err$location = err.location;
  first_line = _err$location.first_line;
  last_line = _err$location.last_line;
  first_column = _err$location.first_column;
  last_column = _err$location.last_column;
  if (last_line == null) {
    last_line = first_line;
  }
  lines = source.split('\n');
  l = lines[first_line];
  lines[first_line] = l.substring(0, first_column) + '\x1B[91m' + l.substring(first_column);
  l = lines[last_line];
  if (first_line === last_line) {
    col = last_column + 6;
  } else {
    col = last_column;
  }
  lines[last_line] = l.substring(0, col) + '\x1B[39m' + l.substring(col);
  lines = lines.slice(first_line, last_line + 1);
  carets = Array(first_column + 1).join(' ') + "\x1B[91m".concat(Array(last_column + 2 - first_column).join('^'), "\x1B[39m");
  lines.push(carets);
  return "".concat(filename, ":").concat(first_line + 1, ":").concat(first_column + 1, ": ").concat(err.name, ": ").concat(err.message, "\n").concat(lines.join('\n'), "\n");
}
var ASTToCode, getAST, _rCFPath, ricffAST;
_rCFPath = function rCFPath(filePath) {
  var fileObj, getFileObj;
  getFileObj = function getFileObj(file) {
    return _objectSpread({
      exists: _fs["default"].existsSync(file)
    }, _path2["default"].parse(file));
  };
  fileObj = getFileObj(filePath);
  if (!fileObj.exists) {
    fileObj = {
      coffee: function () {
        var _path;
        _path = _path2["default"].format({
          root: _path2["default"].join(fileObj.root, fileObj.dir + '/'),
          name: fileObj.base,
          ext: '.coffee'
        });
        return {
          path: _path,
          obj: getFileObj(_path)
        };
      }(),
      js: function () {
        var _path;
        _path = _path2["default"].format({
          root: _path2["default"].join(fileObj.root, fileObj.dir + '/'),
          name: fileObj.base,
          ext: '.js'
        });
        return {
          path: _path,
          obj: getFileObj(_path)
        };
      }()
    };
    if (fileObj.coffee.obj.exists) {
      return _rCFPath(fileObj.coffee.path);
    }
    if (fileObj.js.obj.exists) {
      return _rCFPath(fileObj.js.path);
    }
    return null;
  } else {
    if (fileObj.ext !== '') {
      return filePath;
    } else {
      return _rCFPath(_path2["default"].join(filePath, 'index.coffee'));
    }
  }
};
ricffAST = function ricffAST(ast) {
  var dirname = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : __dirname;
  (0, _traverse["default"])(ast, {
    ImportDeclaration: function ImportDeclaration(_ast) {
      var currentNode, fileParseObj, filePath;
      currentNode = _ast.node;
      fileParseObj = _path2["default"].parse(currentNode.source.value);
      if (fileParseObj.root === '' && fileParseObj.dir === '') {
        return;
      }
      filePath = _rCFPath(_path2["default"].join(dirname, currentNode.source.value));
      if (filePath != null) {
        _ast.node.source = (0, _types.StringLiteral)(filePath);
      }
    }
  });
  return ast;
};
getAST = function getAST(source) {
  return (0, _babylon.parse)(source, {
    sourceType: 'module',
    plugins: ['objectRestSpread', 'asyncGenerators']
  });
};
ASTToCode = function ASTToCode(ast) {
  var es = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;
  var opts = arguments.length > 2 ? arguments[2] : undefined;
  if (es === false) {
    return (0, _cfx["default"])(ast, opts);
  } else {
    return (0, _generator["default"])(ast).code;
  }
};
var sourceMap;
sourceMap = function sourceMap(out) {
  if (out.v3SourceMap) {
    return JSON.parse(out.v3SourceMap);
  } else {
    return null;
  }
};
var coffee2 = function coffee2() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var filter;
  if (opts.bare == null) {
    opts.bare = true;
  }
  if (opts.sourceMap == null) {
    opts.sourceMap = true;
  }
  filter = (0, _rollupPluginutils.createFilter)(opts.include, opts.exclude);
  return {
    name: 'coffee2',
    transform: function transform(code, id) {
      var _code, err, out;
      if (!filter(id)) {
        return {
          code: code
        };
      }
      if (_path2["default"].extname(id) !== '.coffee') {
        return {
          code: code
        };
      }
      try {
        out = _coffeescript["default"].compile(code, {
          filename: id,
          bare: opts.bare,
          sourceMap: opts.sourceMap
        });
      } catch (error) {
        err = error;
        if (err.location != null) {
          err.formattedMessage = formatError(code, id, err);
        }
        throw err;
      }
      _code = ASTToCode(ricffAST(getAST(out.js), _path2["default"].dirname(id)));
      return {
        code: _code,
        map: sourceMap(out)
      };
    }
  };
};
exports.coffee2 = coffee2;
var babel = function babel() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var filter;
  filter = (0, _rollupPluginutils.createFilter)(opts.include, opts.exclude);
  return {
    name: 'babel',
    transform: function transform(code, id) {
      var _code;
      if (!filter(id)) {
        return {
          code: code
        };
      }
      if (_path2["default"].extname(id) !== '.js') {
        return {
          code: code
        };
      }
      _code = ASTToCode(getAST(code), false, opts.opts);
      return _code;
    }
  };
};
exports.babel = babel;
var tasks;
exports.tasks = tasks;
exports.tasks = tasks = {
  clean: function clean(_dirs) {
    return function (cb) {
      var dirs;
      dirs = Array.isArray(_dirs) ? _dirs : [_dirs];
      return (0, _del["default"])(dirs, cb);
    };
  },
  buildEs: function buildEs(gulp, _ref) {
    var index = _ref.index,
        dest = _ref.dest;
    return function () {
      return gulp.src(index).pipe((0, _gulpBetterRollup["default"])({
        plugins: [coffee2({
          bare: true,
          sourceMap: true
        }), (0, _rollupPluginCleanup["default"])()]
      }, 'es')).pipe((0, _gulpRename["default"])('index.js')).pipe(gulp.dest(dest));
    };
  },
  buildCjs: function buildCjs(gulp, _ref2) {
    var index = _ref2.index,
        dest = _ref2.dest;
    return function () {
      return gulp.src(index).pipe((0, _gulpBetterRollup["default"])({
        plugins: [babel(), (0, _rollupPluginCleanup["default"])()]
      }, 'cjs')).pipe((0, _gulpRename["default"])('index.js')).pipe(gulp.dest(dest));
    };
  },
  build: function build(series) {
    return series('buildEs', 'buildCjs');
  },
  rebuild: function rebuild(series) {
    return series('clean', 'build');
  }
};
var gulp = function gulp(_ref3) {
  var gulp = _ref3.gulp,
      series = _ref3.series,
      join = _ref3.join;
  gulp.task('clean', tasks.clean(join('../dist')));
  gulp.task('buildEs', tasks.buildEs(gulp, {
    index: join('../src/index.coffee'),
    dest: join('../dist/es')
  }));
  gulp.task('buildCjs', tasks.buildCjs(gulp, {
    index: join('../dist/es/index.js'),
    dest: join('../dist')
  }));
  gulp.task('build', tasks.build(series));
  return gulp.task('rebuild', tasks.rebuild(series));
};
exports.task = gulp;
